---
layout: post
title: "Apache Arrow 0.6.0 Release"
date: "2017-08-16 00:00:00 -0400"
author: wesm
categories: [release]
---
<!--
{% comment %}
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to you under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
{% endcomment %}
-->

The Apache Arrow team is pleased to announce the 0.6.0 release. It includes
[**90 resolved JIRAs**][1] with the new Plasma shared memory object store, and
improvements and bug fixes to the various language implementations. The Arrow
memory format remains stable since the 0.3.x release.

See the [Install Page][2] to learn how to get the libraries for your
platform. The [complete changelog][5] is also available.

## Plasma Shared Memory Object Store

This release includes the [Plasma Store][7], which you can read more about in
the linked blog post. This system was originally developed as part of the [Ray
Project][8] at the [UC Berkeley RISELab][9]. We recognized that Plasma would be
highly valuable to the Arrow community as a tool for shared memory management
and zero-copy deserialization. Additionally, we believe we will be able to
develop a stronger software stack through sharing of IO and buffer management
code.

The Plasma store is a server application which runs as a separate process. A
reference C++ client, with Python bindings, is made available in this
release. Clients can be developed in Java or other languages in the future to
enable simple sharing of complex datasets through shared memory.

## Arrow Format Addition: Map type

We added a Map logical type to represent ordered and unordered maps
in-memory. This corresponds to the `MAP` logical type annotation in the Parquet
format (where maps are represented as repeated structs).

Map is represented as a list of structs. It is the first example of a logical
type whose physical representation is a nested type. We have not yet created
implementations of Map containers in any of the implementations, but this can
be done in a future release.

As an example, the Python data:

```
data = [{'a': 1, 'bb': 2, 'cc': 3}, {'dddd': 4}]
```

Could be represented in an Arrow `Map<String, Int32>` as:

```
Map<String, Int32> = List<Struct<keys: String, values: Int32>>
  is_valid: [true, true]
  offsets: [0, 3, 4]
  values: Struct<keys: String, values: Int32>
    children:
      - keys: String
          is_valid: [true, true, true, true]
          offsets: [0, 1, 3, 5, 9]
          data: abbccdddd
      - values: Int32
          is_valid: [true, true, true, true]
          data: [1, 2, 3, 4]
```
## Python Changes

Some highlights of Python development outside of bug fixes and general API
improvements include:

* New `strings_to_categorical=True` option when calling `Table.to_pandas` will
  yield pandas `Categorical` types from Arrow binary and string columns
* Expanded Hadoop Filesystem (HDFS) functionality to improve compatibility with
  Dask and other HDFS-aware Python libraries.
* s3fs and other Dask-oriented filesystems can now be used with
  `pyarrow.parquet.ParquetDataset`
* More graceful handling of pandas's nanosecond timestamps when writing to
  Parquet format. You can now pass `coerce_timestamps='ms'` to cast to
  milliseconds, or `'us'` for microseconds.

## Toward Arrow 1.0.0 and Beyond

We are still discussing the roadmap to 1.0.0 release on the [developer mailing
list][6]. The focus of the 1.0.0 release will likely be memory format stability
and hardening integration tests across the remaining data types implemented in
Java and C++. Please join the discussion there.

[1]: https://issues.apache.org/jira/issues/?jql=project%20%3D%20ARROW%20AND%20status%20in%20(Resolved%2C%20Closed)%20AND%20fixVersion%20%3D%200.6.0
[2]: http://arrow.apache.org/install
[3]: http://github.com/apache/parquet-cpp
[5]: http://arrow.apache.org/release/0.6.0.html
[6]: http://mail-archives.apache.org/mod_mbox/arrow-dev/
[7]: http://arrow.apache.org/blog/2017/08/08/plasma-in-memory-object-store/
[8]: https://ray-project.github.io/ray/
[9]: https://rise.cs.berkeley.edu/